package org.gcube.vremanagement;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.collections.bag.SynchronizedSortedBag;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class XmlManager implements VreRemPubInterface {
	private String filePath;
	private HashMap<String, Gateway> gatewaysMap;

	public XmlManager(String filePath) {
		this.filePath = filePath;
		this.gatewaysMap = new HashMap<>();
	}

	@Override
	public void parse() {
		try {
			File xmlFile = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(xmlFile);
			doc.getDocumentElement().normalize();
			NodeList gatewayList = doc.getElementsByTagName("gateways");
			//NodeList gatewayList = gatewayNodes.item(0).getChildNodes();

			//iterate over gateways child
			for(int k =0; k<gatewayList.getLength(); k++)
			{
				Node gatewayNodeT = gatewayList.item(k);
				NodeList gatewayNodeTC = gatewayList.item(k).getChildNodes();
				for(int kk=0; kk<gatewayNodeTC.getLength(); kk++)
				{
					Node gateTest = gatewayNodeTC.item(kk);
					if(gateTest.getNodeType() == Node.ELEMENT_NODE)
					{
						Element gateTestElem = (Element)gateTest;
						String gatewayName = gateTestElem.getElementsByTagName("name").item(0).getTextContent().trim();
						String gatewayDescription = gateTestElem.getElementsByTagName("description").item(0)
								.getTextContent().trim();
						
						//System.out.println("Gate Name"+gatewayName);
						
						Gateway gateway = new Gateway(gatewayName, gatewayDescription);
						
						NodeList vosListT = gateTestElem.getElementsByTagName("vos");
						
						
						for (int j = 0; j < vosListT.getLength(); j++) {
							NodeList vosListC = vosListT.item(j).getChildNodes();
							for (int jj=0; jj < vosListC.getLength();jj++)
							{
							
								Node vosListCE = vosListC.item(jj);
								if (vosListCE.getNodeType() == Node.ELEMENT_NODE) {
								Element vosElement = (Element) vosListCE;
								
								
								
								
								String voKey = vosElement.getElementsByTagName("key").item(0).getTextContent().trim();
								String voName = vosElement.getElementsByTagName("name").item(0).getTextContent().trim();
								String voScope = vosElement.getElementsByTagName("scope").item(0).getTextContent().trim();
										
								Vos vos = new Vos(voKey,voScope, voName);
								
								//System.out.println("VO Name:"+voName);
								
								NodeList vreListT = vosElement.getElementsByTagName("vres");
								
								for (int x=0; x<vreListT.getLength();x++)
								{
									NodeList vreListC = vreListT.item(x).getChildNodes();
									
									for(int xx=0; xx<vreListC.getLength();xx++)
									{
										Node vreListCE = vreListC.item(xx);
										if(vreListCE.getNodeType()==Node.ELEMENT_NODE)
										{
											Element vreEntryElement = (Element) vreListCE;
											String vreKey = vreEntryElement.getElementsByTagName("key").item(0)
													.getTextContent().trim();
											String vreName = vreEntryElement.getElementsByTagName("name")
													.item(0).getTextContent().trim();
											String vreDescription = vreEntryElement
													.getElementsByTagName("description").item(0)
													.getTextContent().trim();
											String vreManager = vreEntryElement.getElementsByTagName("managers")
													.item(0).getTextContent().trim();
											String vreStartDate = vreEntryElement
													.getElementsByTagName("startdate").item(0).getTextContent().trim();
											String vreEndDate = vreEntryElement.getElementsByTagName("enddate")
													.item(0).getTextContent().trim();

											// Create a new Vres object and add it to the Vos's vreList
											Vres vres = new Vres(vreKey, vreName, vreDescription, vreManager,
													vreStartDate, vreEndDate);
											
											//System.out.println("VRE Name:"+vreName);
											
											vos.addVres(vres);
											
										}
									}
								}
								gateway.addVos(vos);
							}
								
							}
						}
						gatewaysMap.put(gatewayName, gateway);
								//getting vo
					}
					
				}
		
		}
		
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	public void getGatewaysInfo() {
		System.out.println("Name of the Gateway:");
		gatewaysMap.keySet().stream().filter(gatewayName -> gatewayName.endsWith("Gateway"))
				.forEach(System.out::println);
	}
	// Other methods...

	@Override
	public void update() {
		// Implement update logic if needed
	}

	@Override
	public void find() {
		// Implement find logic if needed
	}

	@Override
	public void read() {
		// Implement read logic if needed
	}

	@Override
	public void write() {
		// Implement write logic if needed
	}

	//public HashMap<String, ArrayList<Gateway>> parseToHashMap() {
//		return gatewaysMap;
//	}
	
	
	public void getVosInfo(String _gateway) {
		System.out.println("VOs Information for Gateway: " + _gateway);
		
		if (gatewaysMap.containsKey(_gateway)) {
			Gateway gatewayItem = gatewaysMap.get(_gateway);
			//Gateway gateway = gatewayList.get(0); // Get the first gateway with the specified name
			ArrayList<Vos> vosList = gatewayItem.getVosList();
		
			
			if (vosList != null) {
				System.out.println("Num of VOs:"+vosList.size());
				for (Vos vos : vosList) {
				    vos.printInfo();
					//System.out.println("Name of VREs: ");
					//ArrayList<Vres> vresList = vos.getVresList();
					//if (vresList != null && !vresList.isEmpty()) {
					//	for (Vres vre : vresList) {
					//		vre.printInfo();
					//	}
					//} else {
				//		System.out.println("No VREs found for this VOS.");
				//	}
				//	System.out.println("----------------------------------------");
				}
			} else {
				System.out.println("No VOS found for this gateway.");
			}
		} else {
			System.out.println("No gateways found with the name: " + _gateway);
		}
	}

	// Method to find a VO within a gateway
	public void findVo(String _gateway, String _vo) {

		if (gatewaysMap.containsKey(_gateway)) 
		{
			Gateway gatewayItem = gatewaysMap.get(_gateway);
			boolean found = false;
		    ArrayList<Vos> vosList = gatewayItem.getVosList();
		    System.out.println("VO LIST length:"+ vosList.size());
			if (vosList != null) {
					for (Vos vos : vosList) {
						System.out.println("VO: "+vos.getName());
						if (vos.getName().equals(_vo)) {
							found = true;
							System.out.println("VO info for " + _vo + " in " + _gateway + ":");
							System.out.println("Name of VOS: " + vos.getName());
						}
				}
				
			}
			if (!found) {
				System.out.println("VO " + _vo + " not found in the gateway " + _gateway + ".");
			}
		} else {
			System.out.println("Gateway " + _gateway + " not found.");
		}
	}

	public void findVre(String _gateway, String _vo, String _vre) {
	    //System.out.println("VRE Information for VO: " + _vo + " in Gateway:\n " + _gateway);
	    boolean vreFound = false; // Flag to track if VRE exists
	    //ArrayList<Gateway> gatewayList = gatewaysMap.get(_gateway);
	    
	    if (gatewaysMap.containsKey(_gateway)) {
	    	
	    	Gateway gatewayItem = gatewaysMap.get(_gateway);
	            ArrayList<Vos> vosList = gatewayItem.getVosList();
	            if (vosList != null) {
	                for (Vos vos : vosList) {
	                    if (vos.getName().equals(_vo)) {
	                        ArrayList<Vres> vresList = vos.getVresList();
	                        if (vresList != null) {
	                            for (Vres vre : vresList) {
	                                if (vre.getName().equals(_vre)) {
	                                    // Print VRE information
	                                    vre.printInfo();
	                                    vreFound = true; // Set flag to true
	                                    // return; // Exit method after printing information
	                                }
	                            }
	                        }
	                    }
	                }
	         }
	    }
	    // If execution reaches here and VRE not found, print message
	    if (!vreFound) {
	        System.out.println("VRE does not exist in the " + _vo + " of the " + _gateway);
	    }
	}

	
// Other methods...
}
